﻿/**********************************************************************
* Method:		UserControls_DisplayTodaysTimetable :: Page_Load
* Description:	This control is used to display the timetable for the 
*				current day
**********************************************************************/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Web.UI;

using umbraco.cms.businesslogic.media;
using umbraco.interfaces;
using umbraco.NodeFactory;

public partial class UserControls_DisplayTodaysTimetable : UserControl {
	// Initialize class variables
	int _nodeID = 0;
	List<Timetable> _timetable = new List<Timetable>();
	string _icon = string.Empty;

	/**********************************************************************
	* Method:		UserControls_DisplayTodaysTimetable :: Page_Load
	* Description:	Event to occur when the page is loaded
	* Inputs:		Standard page load event arguements
	* Outputs:		None
	**********************************************************************/
	protected void Page_Load ( object sender, EventArgs e ) {
		// Initialize method variables
		DateTime today = DateTime.Now;
		INode topPage = new Node();

		// Go to level 1 as top level menu
		while (topPage.Parent != null) {
			topPage =							topPage.Parent;
		}

		LoopNodes(topPage);
		
		// Get the day of the week
		litDay.Text =									today.DayOfWeek.ToString().ToUpper() + "S";

		// Get the times for the current day and sort them by time
		IEnumerable<Timetable> tbl =					from ttbl in _timetable
														where ttbl.Day == today.DayOfWeek.ToString()
														orderby ttbl.Time
														select ttbl;

		litPrograms.Text =								"<table>";

		// Loop though the sorted times and display them
		foreach (Timetable tt in tbl) {
			litPrograms.Text +=							"<tr>";
			litPrograms.Text +=							"<td width='110'>" + tt.Time.ToString("h.mmtt").ToLower() + "<br />" + tt.Studio + "</td>";
			litPrograms.Text +=							"<td><img src='" +  tt.Icon + "' alt='' style='width:85%;height:85%;' /></td>";
			litPrograms.Text +=							"</tr>";
		}

		litPrograms.Text +=								"</table>";
	}

	/**********************************************************************
	* Method:		UserControls_DisplayTodaysTimetable :: LoopNodes
	* Description:	This method is used to loop through all the nodes
	* Inputs:		node - the node to loop though
	* Outputs:		None
	**********************************************************************/
	private void LoopNodes(INode node) {
		// Loop though each of the child nodes
		foreach(Node n in node.ChildrenAsList) {
			// See if the current node is a program 
			if (n.NodeTypeAlias == "program") {
				_nodeID = n.Id;

				if (n.GetProperty ( "programIcon" ) != null && n.GetProperty ( "programIcon" ).Value != string.Empty) {
					var image = new Media ( Int32.Parse ( n.GetProperty ( "programIcon" ).Value ) );
					var imageUrl = image.getProperty ( "umbracoFile" ).Value.ToString ();
					_icon = imageUrl.Substring(imageUrl.IndexOf("'") + 1, imageUrl.LastIndexOf("'") - imageUrl.IndexOf("'") - 1);
				} else {
					_icon = string.Empty;
				}
			}

			// See if the current node is a program time
			if (n.NodeTypeAlias == "programTIme") {
				// Add the node details to the list
				_timetable.Add(new Timetable(n.GetProperty("programTimeDay").Value, _icon, n.GetProperty("programTimeStudio").Value, Convert.ToDateTime(n.GetProperty("programTime").Value), _nodeID));
			}

			// See if the node has child items
			if (n.ChildrenAsList.Count() > 0) {
				LoopNodes(n);
			}
		}
	}
}